/*
 * Copyright 2010 by Kappich Systemberatung Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.data;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.TimeAttributeType;
import de.bsvrz.pua.prot.util.ExpressionResult;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @author Kappich Systemberatung
 * @version $Revision: 0000 $
 */
class ProtocolDataTimeValue extends ProtocolDataUnscaledValue implements Data.TimeValue {

	/**
	 * Handelt es sich um eine absolute Zeitabgabe?
	 */
	private boolean _isAbsolute = false;

	private static final DateFormat _absoluteMillisecondsFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

	/**
	 * Erstellt ein neue Zeit-Ansicht auf ein Datum
	 * @param attributeProtocolData Datum
	 */
	public ProtocolDataTimeValue(final AttributeProtocolData attributeProtocolData) {
		super(attributeProtocolData);
		final AttributeType type = attributeProtocolData.getAttributeType();
		if(type instanceof TimeAttributeType) {
			final TimeAttributeType timeAttributeType = (TimeAttributeType)type;
			_isAbsolute = !timeAttributeType.isRelative();
		}
	}

	public long getSeconds() {
		return Math.round(longValue() / 1000.0);
	}

	public long getMillis() {
		return longValue();
	}

	@Override
	public String getValueText() {
		try {
			if(_isAbsolute) {
				return _absoluteMillisecondsFormat.format(new Date(getMillis()));
			}
			return formatRelativeTime();
		}
		catch(NumberFormatException ignored) {
			return _data.getInternalValue().toString();
		}
	}

	private String formatRelativeTime() {
		final StringBuilder text = new StringBuilder();
		long val = getMillis();
		//Beispiel= "234 Tage 12 Stunden 34 Minuten 33 Sekunden 443 Millisekunden"
		final int millis = (int)(val % 1000);
		val /= 1000;
		final int seconds = (int)(val % 60);
		val /= 60;
		final int minutes = (int)(val % 60);
		val /= 60;
		final int hours = (int)(val % 24);
		val /= 24;
		final long days = val;
		if(days != 0) {
			if(days == 1) {
				text.append("1 Tag ");
			}
			else if(days == -1) {
				text.append("-1 Tag ");
			}
			else {
				text.append(days).append(" Tage ");
			}
		}
		if(hours != 0) {
			if(hours == 1) {
				text.append("1 Stunde ");
			}
			else if(hours == -1) {
				text.append("-1 Stunde ");
			}
			else {
				text.append(hours).append(" Stunden ");
			}
		}
		if(minutes != 0) {
			if(minutes == 1) {
				text.append("1 Minute ");
			}
			else if(minutes == -1) {
				text.append("-1 Minute ");
			}
			else {
				text.append(minutes).append(" Minuten ");
			}
		}
		if(seconds != 0 || (days == 0 && hours == 0 && minutes == 0 && millis == 0)) {
			if(seconds == 1) {
				text.append("1 Sekunde ");
			}
			else if(seconds == -1) {
				text.append("-1 Sekunde ");
			}
			else {
				text.append(seconds).append(" Sekunden ");
			}
		}
		if(millis != 0) {
			if(millis == 1) {
				text.append("1 Millisekunde ");
			}
			else if(millis == -1) {
				text.append("-1 Millisekunde ");
			}
			else {
				text.append(millis).append(" Millisekunden ");
			}
		}
		text.setLength(text.length() - 1);
		return text.toString();
	}

	@Override
	public String getSuffixText() {
		return "";
	}

	public void setSeconds(final long seconds) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}

	public void setMillis(final long milliSeconds) {
		throw new UnsupportedOperationException(ProtocolData.MESSAGE_NO_CHANGE);
	}
}
