/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.pua.prot.
 * 
 * de.bsvrz.pua.prot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pua.prot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pua.prot.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bsvrz.pua.prot.processing.onlinebuffer;

import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.pua.prot.processing.ValueProvider;
import de.bsvrz.sys.funclib.losb.datk.ProtocolType;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

/**
 * Basisklasse für reale Elemente für den Online-Puffer. aufgerufen werden.
 *
 * @author beck et al. projects GmbH
 * @author Martin Hilgers
 * @version $Revision$ / $Date$ / ($Author$)
 */
public abstract class RealOnlineElement extends ValueProvider {

	/** Receiver der die Daten bereitstellt. */
	private OnlineDataReceiver _receiver;

	/** Objekt das die Empfangsanmeldung übernimmt. */
	private OnlineDataReceiverSet _receiverSet;

	/**
	 * @param receiverSet   Objekt das die Empfangsanmeldung übernimmt.
	 * @param protocolType  Protokollart
	 * @param isShown       Wird das Element im Spalten-Bereich verwendet? <code>true</code>Element wird im Spalten Bereich verwendet.
	 * @param padData       <code>true</code> falls Daten aufgefüllt werden dürfen.
	 */
	public RealOnlineElement(OnlineDataReceiverSet receiverSet, ProtocolType protocolType, boolean isShown, boolean padData) {
		super(protocolType, isShown, padData);
		_receiver = null;
		this._receiverSet = receiverSet;
	}

	/**
	 * Setzt den Receiver.
	 *
	 * @param receiver Empfängerobjekt für die Onlinedaten.
	 */
	public void setReceiver(OnlineDataReceiver receiver) {
		this._receiver = receiver;
	}

	/** @see de.bsvrz.pua.prot.processing.ValueProvider#updateDataInformation() */
	@Override
    public boolean updateDataInformation() {
		if(value != null) {
			dataInformation = dataInformation.createDataInformation(value, false);
			return dataInformation.insertEmpty();
		}
		else {
			return false;
		}
	}

	/**
	 * XXX COMMENT CHANGE
	 *
	 * @see de.bsvrz.pua.prot.processing.ValueProvider#executeTake(long)
	 */
	@Override
    public boolean executeTake(long timeout) throws FailureException, InterruptedException {

		value = _receiver.take(timeout);
		return (value != null);
	}

	/** Bricht die Datensammlung ab. Meldet sich als Empfänger beim Datenverteiler ab. */
	@Override
    public void abort() {
		if(_receiver != null) _receiver.unsubscribe();
	}

	/**
	 * Liefert immmer <code>true</code> zurück.
	 *
	 * @return <code>true</code>
	 *
	 * @see de.bsvrz.pua.prot.processing.ValueProvider#hasNext()
	 */
	@Override
    public boolean hasNext() {
		return true;
	}

	/**
	 * Zeigt auf welche Daten sich das Objekt angemeldet hat.
	 *
	 * @return Datenart.
	 */
	protected ReceiveOptions getReceiveOptions() {
		if(_receiver != null) {
			return _receiverSet.getReceiveOptions();
		}
		else {
			return null;
		}
	}
}
